/* A Bison parser, made by GNU Bison 2.7.12-4996.  */

/* Bison interface for Yacc-like parsers in C
   
      Copyright (C) 1984, 1989-1990, 2000-2013 Free Software Foundation, Inc.
   
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

/* As a special exception, you may create a larger work that contains
   part or all of the Bison parser skeleton and distribute that work
   under terms of your choice, so long as that work isn't itself a
   parser generator using the skeleton or a modified version thereof
   as a parser skeleton.  Alternatively, if you modify or redistribute
   the parser skeleton itself, you may (at your option) remove this
   special exception, which will cause the skeleton and the resulting
   Bison output files to be licensed under the GNU General Public
   License without this special exception.
   
   This special exception was added by the Free Software Foundation in
   version 2.2 of Bison.  */

#ifndef YY_YY_ATTACK_PARSER_H_INCLUDED
# define YY_YY_ATTACK_PARSER_H_INCLUDED
/* Enabling traces.  */
#ifndef YYDEBUG
# define YYDEBUG 0
#endif
#if YYDEBUG
extern int yydebug;
#endif

/* Tokens.  */
#ifndef YYTOKENTYPE
# define YYTOKENTYPE
   /* Put the tokens into the symbol table, so that GDB and other debuggers
      know about them.  */
   enum yytokentype {
     IPv4 = 258,
     IPv6 = 259,
     HOSTADDR = 260,
     WORD = 261,
     INTEGER = 262,
     SYSLOG_BANNER_PID = 263,
     LAST_LINE_REPEATED_N_TIMES = 264,
     SYSLOG_BANNER = 265,
     TIMESTAMP_SYSLOG = 266,
     TIMESTAMP_ISO8601 = 267,
     TIMESTAMP_TAI64 = 268,
     AT_TIMESTAMP_TAI64 = 269,
     METALOG_BANNER = 270,
     SSH_INVALUSERPREF = 271,
     SSH_NOTALLOWEDPREF = 272,
     SSH_NOTALLOWEDSUFF = 273,
     SSH_LOGINERR_PREF = 274,
     SSH_LOGINERR_SUFF = 275,
     SSH_LOGINERR_PAM = 276,
     SSH_VIA = 277,
     SSH_REVERSEMAP_PREF = 278,
     SSH_REVERSEMAP_SUFF = 279,
     SSH_NOIDENTIFSTR = 280,
     SSH_BADPROTOCOLIDENTIF = 281,
     SSH_BADPROTOCOLIDENTIF_SUFF = 282,
     SSH_DISCONNECT_PREF = 283,
     SSH_PREAUTH_SUFF = 284,
     DOVECOT_IMAP_LOGINERR_PREF = 285,
     DOVECOT_IMAP_LOGINERR_SUFF = 286,
     UWIMAP_LOGINERR = 287,
     CYRUSIMAP_SASL_LOGINERR_PREF = 288,
     CYRUSIMAP_SASL_LOGINERR_SUFF = 289,
     CUCIPOP_AUTHFAIL = 290,
     EXIM_ESMTP_AUTHFAIL_PREF = 291,
     EXIM_ESMTP_AUTHFAIL_SUFF = 292,
     SENDMAIL_RELAYDENIED_PREF = 293,
     SENDMAIL_RELAYDENIED_SUFF = 294,
     POSTFIX_SASL_LOGINERR_PREF = 295,
     POSTFIX_SASL_LOGINERR_SUFF = 296,
     FREEBSDFTPD_LOGINERR_PREF = 297,
     FREEBSDFTPD_LOGINERR_SUFF = 298,
     PROFTPD_LOGINERR_PREF = 299,
     PROFTPD_LOGINERR_SUFF = 300,
     PUREFTPD_LOGINERR_PREF = 301,
     PUREFTPD_LOGINERR_SUFF = 302,
     VSFTPD_LOGINERR_PREF = 303,
     VSFTPD_LOGINERR_SUFF = 304
   };
#endif
/* Tokens.  */
#define IPv4 258
#define IPv6 259
#define HOSTADDR 260
#define WORD 261
#define INTEGER 262
#define SYSLOG_BANNER_PID 263
#define LAST_LINE_REPEATED_N_TIMES 264
#define SYSLOG_BANNER 265
#define TIMESTAMP_SYSLOG 266
#define TIMESTAMP_ISO8601 267
#define TIMESTAMP_TAI64 268
#define AT_TIMESTAMP_TAI64 269
#define METALOG_BANNER 270
#define SSH_INVALUSERPREF 271
#define SSH_NOTALLOWEDPREF 272
#define SSH_NOTALLOWEDSUFF 273
#define SSH_LOGINERR_PREF 274
#define SSH_LOGINERR_SUFF 275
#define SSH_LOGINERR_PAM 276
#define SSH_VIA 277
#define SSH_REVERSEMAP_PREF 278
#define SSH_REVERSEMAP_SUFF 279
#define SSH_NOIDENTIFSTR 280
#define SSH_BADPROTOCOLIDENTIF 281
#define SSH_BADPROTOCOLIDENTIF_SUFF 282
#define SSH_DISCONNECT_PREF 283
#define SSH_PREAUTH_SUFF 284
#define DOVECOT_IMAP_LOGINERR_PREF 285
#define DOVECOT_IMAP_LOGINERR_SUFF 286
#define UWIMAP_LOGINERR 287
#define CYRUSIMAP_SASL_LOGINERR_PREF 288
#define CYRUSIMAP_SASL_LOGINERR_SUFF 289
#define CUCIPOP_AUTHFAIL 290
#define EXIM_ESMTP_AUTHFAIL_PREF 291
#define EXIM_ESMTP_AUTHFAIL_SUFF 292
#define SENDMAIL_RELAYDENIED_PREF 293
#define SENDMAIL_RELAYDENIED_SUFF 294
#define POSTFIX_SASL_LOGINERR_PREF 295
#define POSTFIX_SASL_LOGINERR_SUFF 296
#define FREEBSDFTPD_LOGINERR_PREF 297
#define FREEBSDFTPD_LOGINERR_SUFF 298
#define PROFTPD_LOGINERR_PREF 299
#define PROFTPD_LOGINERR_SUFF 300
#define PUREFTPD_LOGINERR_PREF 301
#define PUREFTPD_LOGINERR_SUFF 302
#define VSFTPD_LOGINERR_PREF 303
#define VSFTPD_LOGINERR_SUFF 304



#if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
typedef union YYSTYPE
{
/* Line 2053 of yacc.c  */
#line 70 "attack_parser.y"

    char *str;
    int num;


/* Line 2053 of yacc.c  */
#line 161 "attack_parser.h"
} YYSTYPE;
# define YYSTYPE_IS_TRIVIAL 1
# define yystype YYSTYPE /* obsolescent; will be withdrawn */
# define YYSTYPE_IS_DECLARED 1
#endif

extern YYSTYPE yylval;

#ifdef YYPARSE_PARAM
#if defined __STDC__ || defined __cplusplus
int yyparse (void *YYPARSE_PARAM);
#else
int yyparse ();
#endif
#else /* ! YYPARSE_PARAM */
#if defined __STDC__ || defined __cplusplus
int yyparse (attack_t *attack);
#else
int yyparse ();
#endif
#endif /* ! YYPARSE_PARAM */

#endif /* !YY_YY_ATTACK_PARSER_H_INCLUDED  */
